@@ -1,5 +1,11 @@
 from .backends.interface import LockedException
-from .cache_utils import CacheDetect, CircuitBreakerOpen, RateLimitException, context_cache_detect  # noqa
+from .cache_utils import (  # noqa
+    CacheDetect,
+    CircuitBreakerOpen,
+    PerfDegradationException,
+    RateLimitException,
+    context_cache_detect,
+)
 from .helpers import add_prefix, at  # noqa
 from .wrapper import Cache  # noqa
 
@@ -10,6 +16,7 @@ fail = cache.fail
 circuit_breaker = cache.circuit_breaker
 early = cache.early
 hit = cache.hit
+dynamic = cache.dynamic
 perf = cache.perf
 locked = cache.locked
 invalidate = cache.invalidate
@@ -17,5 +24,5 @@ invalidate = cache.invalidate
 
 mem = Cache()
 mem.setup(
-    "mem://?check_interval=1", size=1_000_000, middlewares=(add_prefix("mem"),),
+    "mem://?check_interval=1", size=1_000_000, middlewares=(add_prefix("mem:"),),
 )  # 1_000_000 * 248(size of small dict) == 31 mb
