@@ -1,6 +1,8 @@
 import asyncio
 import datetime
+import gc
 import re
+import sys
 from typing import Any, Optional, Tuple, Union
 
 from .interface import Backend
@@ -11,6 +13,7 @@ __all__ = ("Memory", "MemoryInterval")
 class Memory(Backend):
     def __init__(self, size: int = 1000):
         self.store = {}
+        self._chans = []
         self.size = size
         self._meta = {}
         self._lock = asyncio.Lock()
@@ -33,6 +36,13 @@ class Memory(Backend):
     async def get_many(self, *keys: str) -> Tuple:
         return tuple([self._get(key) for key in keys])
 
+    async def keys_match(self, pattern: str):
+        pattern = pattern.replace("*", ".*")
+        regexp = re.compile(pattern)
+        for key in dict(self.store):
+            if regexp.fullmatch(key):
+                yield key
+
     async def incr(self, key: str):
         value = int(self._get(key) or 0) + 1
         self._set(key=key, value=value)
@@ -44,6 +54,7 @@ class Memory(Backend):
 
     def _delete(self, key: str) -> bool:
         if key in self.store:
+            self._notify("delete", key)
             del self.store[key]
             if key in self._meta:
                 del self._meta[key]
@@ -51,11 +62,8 @@ class Memory(Backend):
         return False
 
     async def delete_match(self, pattern: str):
-        pattern = pattern.replace("*", "[^:]+")
-        regexp = re.compile(pattern)
-        for key in dict(self.store):
-            if regexp.fullmatch(key):
-                await self.delete(key)
+        async for key in self.keys_match(pattern):
+            await self.delete(key)
 
     async def expire(self, key: str, timeout: float):
         value = self._get(key)
@@ -63,12 +71,17 @@ class Memory(Backend):
             return
         self._set(key=key, value=value, expire=timeout)
 
-    async def ping(self, message: Optional[str] = None):
+    async def get_expire(self, key: str) -> int:
+        return -1
+
+    async def ping(self, message: Optional[bytes] = None):
+        self._notify("ping", message)
         return b"PONG" if message is None else message
 
     def _set(self, key: str, value: Any, expire: Optional[float] = None):
         if len(self.store) > self.size:
             return
+        self._notify("set", key)
         self.store[key] = value
 
         if expire is not None and expire > 0.0:
@@ -79,6 +92,7 @@ class Memory(Backend):
             self._meta[key] = loop.call_later(expire, self._delete, key)
 
     def _get(self, key: str) -> Optional[Any]:
+        self._notify("get", key)
         return self.store.get(key, None)
 
     async def set_lock(self, key: str, value, expire):
@@ -97,6 +111,28 @@ class Memory(Backend):
     async def unlock(self, key, value) -> bool:
         return self._delete(key)
 
+    async def get_size(self, key: str) -> int:
+        if key in self.store:
+            return _get_obj_size(self.store[key])
+        return 0
+
+    def _notify(self, cmd, key):
+        for queue in self._chans:
+            queue.put_nowait((cmd, key))
+
+    async def listen(self, pattern: str, *cmds, reader=None):
+        queue = asyncio.Queue(maxsize=0)
+        pattern = pattern.replace("*", ".*")
+        regexp = re.compile(pattern)
+        self._chans.append(queue)
+        try:
+            while True:
+                cmd, key = await queue.get()
+                if cmd in cmds and regexp.fullmatch(key):
+                    reader(cmd, key)
+        finally:
+            self._chans.remove(queue)
+
 
 class MemoryInterval(Memory):
     def __init__(self, size: int = 1000, check_interval: float = 1.0):
@@ -118,11 +154,42 @@ class MemoryInterval(Memory):
         if expiration and datetime.datetime.utcnow() > expiration:
             await self.delete(key)
             return None
-
+        self._notify("get", key)
         return self.store.get(key, None)
 
     def _set(self, key: str, value: Any, expire: Optional[float] = None) -> None:
+        self._notify("set", key)
         self.store[key] = value
 
         if expire is not None and expire > 0.0:
             self._meta[key] = datetime.datetime.utcnow() + datetime.timedelta(seconds=expire)
+
+    async def get_expire(self, key: str) -> int:
+        if key in self._meta:
+            return abs(int((datetime.datetime.utcnow() - self._meta[key]).total_seconds()))
+        return -1
+
+
+def _get_obj_size(obj) -> int:
+    marked = {id(obj)}
+    obj_q = [obj]
+    size = 0
+
+    while obj_q:
+        size += sum(map(sys.getsizeof, obj_q))
+
+        # Lookup all the object referred to by the object in obj_q.
+        # See: https://docs.python.org/3.7/library/gc.html#gc.get_referents
+        all_refr = ((id(o), o) for o in gc.get_referents(*obj_q))
+
+        # Filter object that are already marked.
+        # Using dict notation will prevent repeated objects.
+        new_refr = {o_id: o for o_id, o in all_refr if o_id not in marked and not isinstance(o, type)}
+
+        # The new obj_q will be the ones that were not marked,
+        # and we will update marked with their ids so we will
+        # not traverse them again.
+        obj_q = new_refr.values()
+        marked.update(new_refr.keys())
+
+    return size
