@@ -1,21 +1,16 @@
 from contextvars import ContextVar
-from typing import Any, Dict
 
 
-def _default_store_condition(result) -> bool:
+def _default_store_condition(result, args, kwargs) -> bool:
     return result is not None
 
 
-def _default_disable_condition(args: Dict[str, Any]) -> bool:
-    return False
-
-
 class CacheDetect:
     def __init__(self):
-        self._value = []
+        self._value = {}
 
-    def set(self, key: str):
-        self._value.append(key)
+    def set(self, key: str, **kwargs):
+        self._value[key] = kwargs
 
     def get(self):
         return self._value
@@ -26,15 +21,18 @@ _var.set(None)
 
 
 class _ContextCacheDetect:
-    def start(self):
+    @staticmethod
+    def start():
         _var.set(CacheDetect())
 
-    def set(self, key: str):
+    @staticmethod
+    def set(key: str, **kwargs):
         var = _var.get()
         if var is not None:
-            var.set(key)
+            var.set(key, **kwargs)
 
-    def get(self):
+    @staticmethod
+    def get():
         var = _var.get()
         if var is not None:
             return var.get()
