@@ -1,9 +1,8 @@
-import asyncio
 from functools import wraps
 from typing import Optional, Tuple, Type, Union
 
 from ..backends.interface import Backend
-from ..key import get_cache_key
+from ..key import get_cache_key, get_cache_key_template, register_template
 from ..typing import FuncArgsType
 from .defaults import CacheDetect, context_cache_detect
 
@@ -29,19 +28,24 @@ def fail(
     """
 
     def _decor(func):
+        _key_template = key
+        if key is None:
+            _key_template = f"{prefix}:{get_cache_key_template(func, func_args=func_args, key=key)}:{ttl}"
+        register_template(func, _key_template)
+
         @wraps(func)
         async def _wrap(*args, _from_cache: CacheDetect = context_cache_detect, **kwargs):
-            _cache_key = prefix + ":" + get_cache_key(func, args, kwargs, func_args, key)
+            _cache_key = get_cache_key(func, _key_template, args, kwargs, func_args)
             try:
                 result = await func(*args, **kwargs)
             except exceptions as exc:
                 cached = await backend.get(_cache_key)
-                if cached:
-                    _from_cache.set(_cache_key)
+                if cached is not None:
+                    _from_cache.set(_cache_key, ttl=ttl, exc=exc)
                     return cached
                 raise exc
             else:
-                asyncio.create_task(backend.set(_cache_key, result, expire=ttl))
+                await backend.set(_cache_key, result, expire=ttl)
                 return result
 
         return _wrap
