@@ -4,14 +4,14 @@ import time
 from collections import deque
 from functools import wraps
 from statistics import mean
-from typing import Any, Callable, Dict, Iterable, Optional
+from typing import Any, Callable, Iterable, Optional
 
 from ..backends.interface import Backend
-from ..key import get_cache_key, get_call_values
+from ..key import get_cache_key, get_cache_key_template, register_template
 from ..typing import FuncArgsType
-from .defaults import CacheDetect, _default_disable_condition, _default_store_condition, context_cache_detect
+from .defaults import CacheDetect, _default_store_condition, context_cache_detect
 
-__all__ = ("hit", "perf", "rate_limit")
+__all__ = ("hit", "perf", "rate_limit", "RateLimitException", "PerfDegradationException")
 
 
 logger = logging.getLogger(__name__)
@@ -21,10 +21,9 @@ def hit(
     backend: Backend,
     ttl: int,
     cache_hits: int,
-    update_before: int = 0,
+    update_before: Optional[int] = None,
     func_args: FuncArgsType = None,
     key: Optional[str] = None,
-    disable: Optional[Callable[[Dict[str, Any]], bool]] = None,
     store: Optional[Callable[[Any], bool]] = None,
     prefix: str = "hit",
 ):
@@ -41,42 +40,47 @@ def hit(
     :param prefix: custom prefix for key, default 'hit'
     """
     store = _default_store_condition if store is None else store
-    disable = _default_disable_condition if disable is None else disable
 
     def _decor(func):
+        _key_template = key
+        if key is None:
+            _key_template = f"{prefix}:{get_cache_key_template(func, func_args=func_args, key=key)}:{ttl}"
+        register_template(func, _key_template)
+
         @wraps(func)
         async def _wrap(*args, _from_cache: CacheDetect = context_cache_detect, **kwargs):
-            if disable(get_call_values(func, args, kwargs, func_args=None)):
-                return await func(*args, **kwargs)
-
-            _cache_key = prefix + ":" + get_cache_key(func, args, kwargs, func_args, key)
+            _cache_key = get_cache_key(func, _key_template, args, kwargs, func_args)
             result = await backend.get(_cache_key)
             hits = await backend.incr(_cache_key + ":counter")
-            if result and hits <= cache_hits:
-                _from_cache.set(_cache_key)
-                if update_before and cache_hits - hits == update_before:
-                    asyncio.create_task(_get_and_save(func, args, kwargs, backend, _cache_key, ttl))
+            if result is not None and hits <= cache_hits:
+                _from_cache.set(_cache_key, ttl=ttl, cache_hits=cache_hits)
+                if update_before is not None and cache_hits - hits <= update_before:
+                    asyncio.create_task(_get_and_save(func, args, kwargs, backend, _cache_key, ttl, store))
                 return result
-            result = await func(*args, **kwargs)
-            if store(result):
-                asyncio.create_task(backend.delete(_cache_key + ":counter"))
-                asyncio.create_task(backend.set(_cache_key, result, expire=ttl))
-            return result
+            return await _get_and_save(func, args, kwargs, backend, _cache_key, ttl, store)
 
         return _wrap
 
     return _decor
 
 
-async def _get_and_save(func, args, kwargs, backend, key, ttl):
+async def _get_and_save(func, args, kwargs, backend, key, ttl, store):
     result = await func(*args, **kwargs)
-    await backend.set(key, result, expire=ttl)
+    if store(result, args, kwargs):
+        await backend.delete(key + ":counter")
+        await backend.set(key, result, expire=ttl)
+
+    return result
 
 
 def _default_perf_condition(current: float, previous: Iterable[float]) -> bool:
     return mean(previous) * 2 < current
 
 
+class PerfDegradationException(Exception):
+    pass
+
+
 def perf(
     backend: Backend,
     ttl: int,
@@ -87,9 +91,9 @@ def perf(
     prefix: str = "perf",
 ):
     """
-    Trace time execution of target and enable cache if it downgrade to given condition
+    Trace time execution of target and throw exception if it downgrade to given condition
     :param backend: cache backend
-    :param ttl: duration in seconds to store a result
+    :param ttl: duration in seconds to lock
     :param func_args: arguments that will be used in key
     :param key: custom cache key, may contain alias to args or kwargs passed to a call
     :param trace_size: the number of calls that are involved
@@ -102,22 +106,23 @@ def perf(
     call_results = deque([], maxlen=trace_size)
 
     def _decor(func):
+        _key_template = key
+        if key is None:
+            _key_template = f"{prefix}:{get_cache_key_template(func, func_args=func_args, key=key)}:{ttl}"
+
         @wraps(func)
-        async def _wrap(*args, _from_cache: CacheDetect = context_cache_detect, **kwargs):
-            _cache_key = prefix + ":" + get_cache_key(func, args, kwargs, func_args, key)
+        async def _wrap(*args, **kwargs):
+            _cache_key = get_cache_key(func, _key_template, args, kwargs, func_args)
             if await backend.is_locked(_cache_key + ":lock"):
-                cached = await backend.get(_cache_key)
-                if cached:
-                    _from_cache.set(_cache_key)
-                    return cached
-            start = time.time()
+                raise PerfDegradationException()
+            start = time.perf_counter()
             result = await func(*args, **kwargs)
-            takes = time.time() - start
+            takes = time.perf_counter() - start
             if len(call_results) == trace_size and perf_condition(takes, call_results):
                 await backend.set_lock(_cache_key + ":lock", value=takes, expire=ttl)
-                await backend.set(_cache_key, result, expire=ttl)
-                return result
-            call_results.append(takes)
+                call_results.clear()
+            else:
+                call_results.append(takes)
             return result
 
         return _wrap
@@ -156,9 +161,12 @@ def rate_limit(
     action = _default_action if action is None else action
 
     def decorator(func):
+        _key_template = f"{prefix}:{get_cache_key_template(func, func_args=func_args)}"
+        register_template(func, _key_template)
+
         @wraps(func)
         async def wrapped_func(*args, **kwargs):
-            _cache_key = prefix + ":" + get_cache_key(func, args, kwargs, func_args)
+            _cache_key = get_cache_key(func, _key_template, args, kwargs, func_args)
 
             requests_count = await backend.incr(key=_cache_key)  # set 1 if not exists
             if requests_count and requests_count > limit:
