@@ -2,13 +2,13 @@ import asyncio
 from unittest.mock import Mock
 
 import pytest
-from cashews.backends.memory import Memory
+from cashews.backends.memory import Backend, Memory
 from cashews.cache_utils.circuit_braker import CircuitBreakerOpen, circuit_breaker
-from cashews.cache_utils.defaults import context_cache_detect
+from cashews.cache_utils.defaults import CacheDetect, context_cache_detect
 from cashews.cache_utils.early import early as early_cache
 from cashews.cache_utils.fail import fail
 from cashews.cache_utils.locked import locked as lock_cache
-from cashews.cache_utils.rate import hit, perf
+from cashews.cache_utils.rate import PerfDegradationException, hit, perf
 from cashews.cache_utils.simple import cache
 
 pytestmark = pytest.mark.asyncio
@@ -43,7 +43,7 @@ async def test_fail_cache_simple(backend):
 
 
 async def test_circuit_breaker_simple(backend):
-    @circuit_breaker(backend, ttl=EXPIRE * 10, errors_rate=0.1, period=1, func_args=())
+    @circuit_breaker(backend, ttl=EXPIRE * 10, errors_rate=5, period=1, func_args=())
     async def func(fail=False):
         if fail:
             raise CustomError()
@@ -77,23 +77,28 @@ async def test_cache_simple(backend):
     assert await func(b"notok") == b"notok"
 
 
-async def test_cache_simple_enable(backend):
-    @cache(backend, ttl=1, key="key", disable=lambda args: args["no_cache"])
-    async def func(resp=b"ok", no_cache=False):
+async def test_cache_simple_key(backend):
+    @cache(backend, ttl=EXPIRE, func_args=("some",))
+    async def func(resp=b"ok", some="err"):
+        return resp
+
+    @cache(backend, ttl=1, func_args=("some",))
+    async def func2(resp=b"ok", some="err"):
         return resp
 
     assert await func() == b"ok"
+    assert await func(b"notok", some="test") == b"notok"
 
     await asyncio.sleep(0)
     assert await func(b"notok") == b"ok"
 
-    assert await func(b"notok", no_cache=True) == b"notok"
+    assert await func2(b"notok") == b"notok"
 
 
 async def test_cache_simple_cond(backend):
     mock = Mock()
 
-    @cache(backend, ttl=EXPIRE, key="key", store=lambda x: x == b"hit")
+    @cache(backend, ttl=EXPIRE, key="key", store=lambda x, *args, **kwargs: x == b"hit")
     async def func(resp=b"ok"):
         mock()
         return resp
@@ -106,7 +111,6 @@ async def test_cache_simple_cond(backend):
     assert mock.call_count == 2
 
     await func(b"hit")
-    await asyncio.sleep(0)  # allow to set cache coroutine work
     assert mock.call_count == 3
     await func(b"hit")
     assert mock.call_count == 3
@@ -140,7 +144,7 @@ async def test_early_cache_parallel(backend):
 
     assert mock.call_count == 1
 
-    for _ in range(5):  # 0.01 (first) + 4 * 0.01 = 0.06 + 0.01(executing) 0.8 will execute
+    for _ in range(4):  # 0.01 (first) + 4 * 0.01 = 0.06 + 0.01(executing) 0.8 will execute
         await asyncio.sleep(0.01)
         await asyncio.gather(*[func() for _ in range(10)])
 
@@ -156,23 +160,32 @@ async def test_early_cache_parallel(backend):
 async def test_lock_cache_parallel(backend):
     mock = Mock()
 
-    @lock_cache(backend, ttl=0.1, key="key", step=0.01)
+    @lock_cache(backend, key="key", step=0.01)
     async def func(resp=b"ok"):
         await asyncio.sleep(0.01)
         mock(resp)
         return resp
 
     for _ in range(4):
+        await asyncio.gather(*[func() for _ in range(10)], return_exceptions=True)
+
+    assert mock.call_count == 4
+
+
+async def test_lock_cache_broken_backend():
+    backend = Mock(wraps=Backend())
+    mock = Mock()
+
+    @lock_cache(backend, key="key", step=0.01)
+    async def func(resp=b"ok"):
         await asyncio.sleep(0.01)
+        mock(resp)
+        return resp
+
+    for _ in range(4):
         await asyncio.gather(*[func() for _ in range(10)])
 
-    assert mock.call_count == 1
-
-    for _ in range(10):
-        await asyncio.sleep(0.01)
-        await asyncio.gather(*[func() for _ in range(10)])
-
-    assert mock.call_count == 2
+    assert mock.call_count == 40
 
 
 async def test_hit_cache(backend):
@@ -193,6 +206,25 @@ async def test_hit_cache(backend):
     assert mock.call_count == 2
 
 
+async def test_hit_cache_early(backend):
+    mock = Mock()
+
+    @hit(backend, ttl=10, cache_hits=1, key="test", update_before=0)
+    async def func(resp=b"ok"):
+        mock(resp)
+        return resp
+
+    assert await func(b"1") == b"1"  # cache
+    assert mock.call_count == 1
+
+    assert await func(b"2") == b"1"  # cache
+    assert mock.call_count == 1
+
+    await asyncio.sleep(0)
+    assert await func(b"3") == b"2"  # cache
+    assert mock.call_count == 2
+
+
 async def test_perf_cache(backend):
     mock = Mock()
 
@@ -204,18 +236,38 @@ async def test_perf_cache(backend):
 
     await asyncio.gather(*[func() for _ in range(10)])
     assert mock.call_count == 10
-    await func(0.015)
+    await func(0.04)
     assert mock.call_count == 11
-    await func(0.04)  # long
-    assert mock.call_count == 12
+    with pytest.raises(PerfDegradationException):
+        await func(0.001)  # long
+        assert mock.call_count == 11
 
     # prev was slow so no hits
-    await asyncio.gather(*[func() for _ in range(1000)])
-    assert mock.call_count == 12
+    with pytest.raises(PerfDegradationException):
+        await asyncio.gather(*[func() for _ in range(1000)])
+
+    assert mock.call_count == 11
     await asyncio.sleep(0.07)
 
     await func(0.009)
-    assert mock.call_count == 13
+    assert mock.call_count == 12
+
+
+async def test_cache_detect_simple(backend):
+    @cache(backend, ttl=EXPIRE, key="key")
+    async def func(resp=b"ok"):
+        return resp
+
+    cache_detect = CacheDetect()
+    assert await func(_from_cache=cache_detect) == b"ok"
+    assert cache_detect.get() == {}
+
+    await asyncio.sleep(0)
+    assert await func(b"notok", _from_cache=cache_detect) == b"ok"
+    assert len(cache_detect.get()) == 1
+    assert list(cache_detect.get().keys()) == [
+        "key",
+    ]
 
 
 async def test_context_cache_detect_simple(backend):
@@ -225,12 +277,12 @@ async def test_context_cache_detect_simple(backend):
 
     context_cache_detect.start()
     assert await func() == b"ok"
-    assert context_cache_detect.get() == []
+    assert context_cache_detect.get() == {}
 
     await asyncio.sleep(0)
     assert await func(b"notok") == b"ok"
     assert len(context_cache_detect.get()) == 1
-    assert context_cache_detect.get() == [
+    assert list(context_cache_detect.get().keys()) == [
         "key",
     ]
 
@@ -239,7 +291,7 @@ async def test_context_cache_detect_simple(backend):
     assert len(context_cache_detect.get()) == 1
 
 
-async def test_context_cache_detect_deap(backend):
+async def test_context_cache_detect_deep(backend):
     @cache(backend, ttl=EXPIRE, key="key1")
     async def func1():
         return 1
@@ -253,7 +305,7 @@ async def test_context_cache_detect_deap(backend):
 
     context_cache_detect.start()
     await func()
-    assert context_cache_detect.get() == []
+    assert context_cache_detect.get() == {}
 
     await asyncio.sleep(0)
     await func()
@@ -277,9 +329,6 @@ async def test_context_cache_detect_context(backend):
         assert len(context_cache_detect.get()) == 0
         await t_func()
         assert len(context_cache_detect.get()) == 1
-        assert context_cache_detect.get() == [
-            t_func._key_template,
-        ]
 
     await backend.set("key1", "test")
     await backend.set("key2", "test")
