@@ -28,7 +28,11 @@ class DummyCache:
     async def get(self, key: str, *args, **kwargs):
         return self.store.get(key, None)
 
-    async def get_many(self, *keys):
+    async def delete(self, *keys):
+        for key in keys:
+            del self.store[key]
+
+    async def mget(self, *keys):
         return [self.store.get(key, None) for key in keys]
 
 
@@ -64,9 +68,7 @@ def _cache():
 async def test_serialize_simple_value(value, cache):
     await cache.set("key", value)
     assert await cache.get("key") == value
-    assert await cache.get_many("key") == [
-        value,
-    ]
+    assert await cache.mget("key") == (value,)
 
 
 @pytest.mark.parametrize("value", (NT(name="test", test="lol"), NT()))
@@ -106,10 +108,12 @@ async def test_unsecure_value(cache):
     cache.store["key"] = b"cos\nsystem\n(S'echo hello world'\ntR."
     with pytest.raises(UnSecureDataError):
         await cache.get("key")
+    assert "key" not in cache.store
 
     cache.store["key"] = b"_cos\nsystem\n(S'echo hello world'\ntR."
     with pytest.raises(UnSecureDataError):
         await cache.get("key")
+    assert "key" not in cache.store
 
 
 async def test_no_value(cache):
@@ -122,11 +126,13 @@ async def test_replace_values(cache):
 
     with pytest.raises(UnSecureDataError):
         await cache.get("replace")
+    assert "replace" not in cache.store
 
 
 async def test_pickle_error_value(cache):
-    cache.store["key"] = cache.get_sign("key", b"no_pickle_data") + b"_" + b"no_pickle_data"
+    cache.store["key"] = cache.get_sign("key", b"no_pickle_data", b"md5") + b"_" + b"no_pickle_data"
     assert await cache.get("key") is None
+    assert "key" not in cache.store
 
 
 async def test_set_no_ser(cache):
